// This unit contains two components, TWDGradientLabel and
// TWDGradientPanel. It uses the GradientFill function in
// the MSIMG32.DLL to draw the gradient providing fast and
// smooth gradients.
//
// Author: Wilco van Driel (wilco1967@hotmail.com)
// Date  : 05 June 2001
// Place : London, UK
//
// The source code to these components is given AS-IS to
// the Delphi community. If you want to use it, do so in
// it's original form or modify it any way you see fit.
//
// The author does not mind getting emails thanking him for
// the code.
unit WDGradient;

interface

uses
  Classes, Controls, ExtCtrls, Graphics, StdCtrls, SysUtils, Windows;

type
  TGradientDirection=(gdHorizontal,gdVertical);

  // Label displaying a gradient background behind the text
  TWDGradientLabel=class(TCustomLabel)
  private
    MyColor1   : TColor;
    MyColor2   : TColor;
    MyDirection: TGradientDirection;
  protected
    procedure    SetColor1(NewColor1: TColor);
    procedure    SetColor2(NewColor2: TColor);
    procedure    SetDirection(NewDirection: TGradientDirection);
  public
    constructor  Create(AOwner: TComponent); override;
    procedure    Paint; override;
  published
    property     Align;
    property     Alignment default taCenter;
    property     Anchors;
    property     AutoSize;
    property     BiDiMode;
    property     Caption;
    property     Color default clBlue;
    property     Color1: TColor read MyColor1 write SetColor1 default clBlue;
    property     Color2: TColor read MyColor2 write SetColor2 default clBlack;
    property     Constraints;
    property     DragCursor;
    property     DragKind;
    property     DragMode;
    property     Enabled;
    property     FocusControl;
    property     Font;
    property     GradientDirection: TGradientDirection read MyDirection write SetDirection;
    property     Layout default tlCenter;
    property     OnClick;
    property     OnContextPopup;
    property     OnDblClick;
    property     OnDragDrop;
    property     OnDragOver;
    property     OnEndDock;
    property     OnEndDrag;
    property     OnMouseDown;
    property     OnMouseMove;
    property     OnMouseUp;
    property     OnStartDock;
    property     OnStartDrag;
    property     ParentBiDiMode;
    property     ParentColor;
    property     ParentFont;
    property     ParentShowHint;
    property     PopupMenu;
    property     ShowAccelChar;
    property     ShowHint;
    property     Visible;
    property     WordWrap;
  end;

  // Label displaying a gradient background
  TWDGradientPanel=class(TCustomPanel)
  private
    MyColor1   : TColor;
    MyColor2   : TColor;
    MyDirection: TGradientDirection;
  protected
    procedure    SetColor1(NewColor1: TColor);
    procedure    SetColor2(NewColor2: TColor);
    procedure    SetDirection(NewDirection: TGradientDirection);
  public
    constructor  Create(AOwner: TComponent); override;
    procedure    Paint; override;
    property     DockManager;
  published
    property     Align;
    property     Alignment default taCenter;
    property     Anchors;
    property     AutoSize;
    property     BevelInner;
    property     BevelOuter;
    property     BevelWidth;
    property     BiDiMode;
    property     BorderStyle;
    property     BorderWidth;
    property     Caption;
    property     Color default clBlue;
    property     Color1: TColor read MyColor1 write SetColor1 default clBlue;
    property     Color2: TColor read MyColor2 write SetColor2 default clBlack;
    property     Constraints;
    property     Ctl3D;
    property     DockSite;
    property     DragCursor;
    property     DragKind;
    property     DragMode;
    property     Enabled;
    property     Font;
    property     FullRepaint;
    property     GradientDirection: TGradientDirection read MyDirection write SetDirection;
    property     Locked;
    property     OnCanResize;
    property     OnClick;
    property     OnConstrainedResize;
    property     OnContextPopup;
    property     OnDblClick;
    property     OnDockDrop;
    property     OnDockOver;
    property     OnDragDrop;
    property     OnDragOver;
    property     OnEndDock;
    property     OnEndDrag;
    property     OnEnter;
    property     OnExit;
    property     OnGetSiteInfo;
    property     OnMouseDown;
    property     OnMouseMove;
    property     OnMouseUp;
    property     OnResize;
    property     OnStartDock;
    property     OnStartDrag;
    property     OnUnDock;
    property     ParentBiDiMode;
    property     ParentColor;
    property     ParentCtl3D;
    property     ParentFont;
    property     ParentShowHint;
    property     PopupMenu;
    property     ShowHint;
    property     TabOrder;
    property     TabStop;
    property     UseDockManager default True;
    property     Visible;
  end;

  // Procedure to register the components
  procedure Register;

implementation

// The definition of the TTriVertex structure in Windows.Pas is
// incorrect, therefore define my own
type
  TTriVertex = packed record
    x    : Longint;
    y    : Longint;
    Red  : SmallInt;
    Green: SmallInt;
    Blue : SmallInt;
    Alpha: SmallInt;
  end;

// Variables used for interfacing to the MSIMG32.DLL
var
  CanGradient   : Boolean;
  GradProc      : function (DC: HDC; var p2: TTriVertex; p3: ULONG; p4: Pointer; p5, p6: ULONG): BOOL; stdcall;
  Img32DLLHandle: Integer;

// Support function drawing a gradient on a canvas
procedure DrawGradient(Canvas: TCanvas;Rect: TRect;Color1: TColor;Color2: TColor;Direction: TGradientDirection);

  // Function to initialise a TTriVertex
  function InitTriVertex(XPos: Integer;YPos: Integer;Color: TColor): TTriVertex;
  begin
    Result.x:=XPos;
    Result.y:=YPos;
    Result.Alpha:=2;
    Result.Red:=(GetRValue(ColorToRGB(Color)) shl 8);
    Result.Green:=(GetGValue(ColorToRGB(Color)) shl 8);
    Result.Blue:=(GetBValue(ColorToRGB(Color)) shl 8);
  end;

var
  GRect : TGradientRect;
  Result: BOOL;
  Vertex: array[0..2] of TTriVertex;
begin
  Result:=False;
  GRect.UpperLeft:=0;
  GRect.LowerRight:=1;
  Vertex[0]:=InitTriVertex(Rect.Left,Rect.Top,Color1);
  Vertex[1]:=InitTriVertex(Rect.Right,Rect.Bottom,Color2);
  case Direction of
    gdHorizontal:
      Result:=GradProc(Canvas.Handle,Vertex[0],2,@GRect,1,GRADIENT_FILL_RECT_H);
    gdVertical:
      Result:=GradProc(Canvas.Handle,Vertex[0],2,@GRect,1,GRADIENT_FILL_RECT_V);
  end;
  if not(Result) then
    raise Exception.Create('GradientFill failed');
end;

// Create an instance of the TWDGradientLabel
constructor TWDGradientLabel.Create(AOwner: TComponent);
begin
  inherited;
  Alignment:=taCenter;
  Color:=clBlue;
  Color1:=clBlue;
  Color2:=clBlack;
  Font.Color:=clWhite;
  Layout:=tlCenter;
  Transparent:=CanGradient;
end;

// Paint a TWDGradientLabel
procedure TWDGradientLabel.Paint;
begin
  if CanGradient then
    DrawGradient(Canvas,ClientRect,Color1,Color2,GradientDirection);

  // Now call the inherited Paint method
  inherited;
end;

// Update Color1
procedure TWDGradientLabel.SetColor1(NewColor1: TColor);
begin
  MyColor1:=NewColor1;
  Invalidate;
end;

// Update Color2
procedure TWDGradientLabel.SetColor2(NewColor2: TColor);
begin
  MyColor2:=NewColor2;
  Invalidate;
end;

// Update gradient direction
procedure TWDGradientLabel.SetDirection(NewDirection: TGradientDirection);
begin
  MyDirection:=NewDirection;
  Invalidate;
end;

// Create an instance of the TWDGradientPanel
constructor TWDGradientPanel.Create(AOwner: TComponent);
begin
  inherited;
  Alignment:=taCenter;
  Color:=clBlue;
  Color1:=clBlue;
  Color2:=clBlack;
  Font.Color:=clWhite;
end;

// Paint a TWDGradientPanel
procedure TWDGradientPanel.Paint;
const
  Alignments: array[TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Rect      : TRect;
  Space     : Integer;
  FontHeight: Integer;
  Flags     : Longint;
begin
  // Call the inherited Paint method
  inherited;
  if CanGradient then
  begin
    // The inherited Paint method has drawn the bevels and filled the
    // background with Color, now draw the Gradient in the space
    // between the bevels
    Space:=0;
    if BevelInner<>bvNone then
      Space:=Space+1;
    if BevelOuter<>bvNone then
      Space:=Space+1;
    Rect.Top:=ClientRect.Top+Space;
    Rect.Left:=ClientRect.Left+Space;
    Rect.Bottom:=ClientRect.Bottom-Space;
    Rect.Right:=ClientRect.Right-Space;
    // Use the DrawGradient function
    DrawGradient(Canvas,Rect,Color1,Color2,GradientDirection);
// Following code is taken from the inherited TCustomPanel.Paint method
// and will paint the caption on top of the gradient
// Copied code starts here >>>
    with Canvas do
    begin
      Brush.Style := bsClear;
      Font := Self.Font;
      FontHeight := TextHeight('W');
      with Rect do
      begin
        Top := ((Bottom + Top) - FontHeight) div 2;
        Bottom := Top + FontHeight;
      end;
      Flags := DT_EXPANDTABS or DT_VCENTER or Alignments[Alignment];
      Flags := DrawTextBiDiModeFlags(Flags);
      DrawText(Handle, PChar(Caption), -1, Rect, Flags);
    end;
// <<< Copied code ends here
  end;
end;

// Set Color1
procedure TWDGradientPanel.SetColor1(NewColor1: TColor);
begin
  MyColor1:=NewColor1;
  Invalidate;
end;

// Set Color2
procedure TWDGradientPanel.SetColor2(NewColor2: TColor);
begin
  MyColor2:=NewColor2;
  Invalidate;
end;

// Set gradient direction
procedure TWDGradientPanel.SetDirection(NewDirection: TGradientDirection);
begin
  MyDirection:=NewDirection;
  Invalidate;
end;

// Register the components
procedure Register;
begin
  RegisterComponents('WD',[TWDGradientLabel]);
  RegisterComponents('WD',[TWDGradientPanel]);
end;

// Unit initialisation
initialization
begin
  CanGradient:=False;
  Img32DLLHandle:=0;
  Img32DLLHandle:=LoadLibrary('MSIMG32.DLL');
  // Could we load the DLL?
  if Img32DLLHandle<>0 then
  begin
    GradProc:=GetProcAddress(Img32DLLHandle,'GradientFill');
    // Could we find the GradientFill method?
    if Assigned(GradProc) then
      // All is fine, we can do gradients
      CanGradient:=True;
  end;
end;

// Clean up the unit
finalization
begin
  // Has the DLL been loaded?
  if Img32DLLHandle<>0 then
    FreeLibrary(Img32DLLHandle);
end;

end.

